﻿using System;
using System.Collections.Generic;
using Microsoft.WowAddonStudio.FrameXml.Components.Support;

namespace Microsoft.WowAddonStudio.FrameXml.Components.Layout
{
	public class FrameXmlLayoutEngine
	{
		private readonly Dictionary<string, List<string>> dependencies = new Dictionary<string, List<string>>();

		/// <summary>
		/// Initializes layout management for the FrameXML control.
		/// </summary>
		/// <param name="control">The FrameXML control.</param>
		public virtual void InitLayout(IFrameXmlControl control)
		{
			if (control == null)
				throw new ArgumentNullException("control");

			var layoutFrameControl = control as LayoutFrameControl;
			if (layoutFrameControl != null && layoutFrameControl.LayoutFrame.Anchors != null)
			{
				// Iterate through the anchors and look for dependencies
				foreach (var anchor in layoutFrameControl.LayoutFrame.Anchors)
				{
					if (anchor.RelativeTo.IsAvailable())
					{
						// The layout frame has an explicit dependency on another frame
						AddDependency(control.Name, control.Parent.ExpandName(anchor.RelativeTo));
					}
					else
					{
						// The layout frame implicitly depends on its parent
						AddDependency(control.Name, control.Parent.Name);
					}
				}
			}
		}

		/// <summary>
		/// Performs layout for the FrameXML control.
		/// </summary>
		/// <param name="control">The FrameXML control.</param>
		/// <param name="affectedProperty">The affected property.</param>
		public virtual void Layout(IFrameXmlControl control, string affectedProperty)
		{
			if (control == null)
				throw new ArgumentNullException("control");

			// Perform the layout for the FrameXML control
			if (control is LayoutFrameControl)
				((LayoutFrameControl) control).Layout(affectedProperty);

			// Perform layout for dependent controls
			if (dependencies.ContainsKey(control.Name))
			{
				foreach (var dependentControlName in dependencies[control.Name])
				{
					// Find the dependent control in the container
					var dependentControl = control.Site.Container.FindFrameXmlControlByName<LayoutFrameControl>(dependentControlName);
					if (dependentControl != null)
					{
						// Trigger layout of dependent control
						Layout(dependentControl, affectedProperty);
					}
				}
			}
		}

		/// <summary>
		/// Removes the layout.
		/// </summary>
		/// <param name="component">The control.</param>
		public virtual void RemoveLayout(IFrameXmlComponent component)
		{
			dependencies.Remove(component.Name);
		}

        /// <summary>
        /// Determines whether this instance [can add dependency] the specified control name.
        /// </summary>
        /// <param name="controlName">Name of the control.</param>
        /// <param name="dependencyName">Name of the dependency.</param>
        /// <returns>
        /// 	<c>true</c> if this instance [can add dependency] the specified control name; otherwise, <c>false</c>.
        /// </returns>
        public bool CanAddDependency(string controlName, string dependencyName)
        {
            if (dependencies.ContainsKey(controlName))
            {
                foreach (string controlDependencyName in dependencies[controlName])
                {
                    if (controlDependencyName == dependencyName)
                        return false;

                    if (!CanAddDependency(controlDependencyName, dependencyName))
                        return false;
                }
            }

            return true;
        }

	    /// <summary>
		/// Adds the dependency.
		/// </summary>
		/// <param name="controlName">Name of the control.</param>
		/// <param name="dependencyName">Name of the dependency.</param>
		protected virtual void AddDependency(string controlName, string dependencyName)
		{
			//Trace.WriteLine(String.Format("AddDependency: {0} => {1}", controlName, dependencyName));

			if (!dependencies.ContainsKey(dependencyName))
				dependencies[dependencyName] = new List<string> {controlName};
			else
			{
				if (!dependencies[dependencyName].Contains(controlName))
					dependencies[dependencyName].Add(controlName);
			}
		}
	}
}