using System;
using System.Runtime.Serialization;
using System.Security.Permissions;

namespace Microsoft.WowAddonStudio.FrameXml.Design.Serialization
{
	/// <summary>
	/// Represents errors that occur during Serialization.
	/// </summary>
	[Serializable]
	public class BadSerializationObjectException : Exception
	{

		/// <summary>
		/// Gets or sets the serialization object.
		/// </summary>
		/// <value>The serialization object.</value>
		public object SerializationObject { get; protected set; }

		/// <summary>
		/// Initializes a new instance of the <see cref="BadSerializationObjectException"/> class.
		/// </summary>
		/// <param name="serializationObject">The <see cref="object"/> parameter.</param>
		public BadSerializationObjectException(object serializationObject)
			: this(serializationObject, null, null)
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="BadSerializationObjectException"/> class
		///  with a specified error message.
		/// </summary>
		/// <param name="serializationObject">The <see cref="object"/> parameter.</param>
		/// <param name="message">The message that describes the error.</param>
		public BadSerializationObjectException(object serializationObject, string message)
			: this(serializationObject, message, null)
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="BadSerializationObjectException"/> class with a specified error message
		/// and a reference to the inner exception that is the cause of this exception.
		/// </summary>
		/// <param name="serializationObject">The <see cref="object"/> parameter.</param>
		/// <param name="message">The message that describes the error.</param>
		/// <param name="innerException">The exception that is the cause of the current exception, or a null reference if no inner exception is specified.</param>
		public BadSerializationObjectException(object serializationObject, string message, Exception innerException)
			: base(message, innerException)
		{
			SerializationObject = serializationObject;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="BadSerializationObjectException"/> class with serialized data.
		/// </summary>
		/// <param name="info">The <see cref="System.Runtime.Serialization.SerializationInfo"/> that holds the serialized object data about the exception being thrown.</param>
		/// <param name="context">The <see cref="System.Runtime.Serialization.StreamingContext"/> that contains contextual information about the source or destination.</param>
		protected BadSerializationObjectException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
			if (info != null)
				SerializationObject = info.GetValue("SerializationObject", typeof(object));
		}

		/// <summary>
		/// When overridden in a derived class, sets the System.Runtime.Serialization.SerializationInfo with information about the exception.
		/// </summary>
		/// <param name="info"></param>
		/// <param name="context"></param>
		[SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter=true)]
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
			info.AddValue("SerializationObject", SerializationObject);
		}

		/// <summary>
		/// Gets a message that describes the current exception.
		/// </summary>
		/// <returns>The error message that explains the reason for the exception, or an empty string("").</returns>
		public override string Message
		{
			get
			{
				string message = base.Message;
				if (InnerException != null)
					message = String.Concat(message, "\n", InnerException.Message);
				return message;
			}
		}
	}
}