using System;
using System.Runtime.Serialization;
using System.Security.Permissions;

namespace Microsoft.WowAddonStudio.FrameXml.Editor.Support
{
	/// <summary>
	/// Represents errors that occur during LuaRuntime execution.
	/// </summary>
	[Serializable]
	public class InvalidWowBasePathException : Exception
	{
		/// <summary>
		/// Gets the name of the runtime which caused the exception.
		/// </summary>
		public string WowBasePath { get; protected set; }

		/// <summary>
		/// Initializes a new instance of the <see cref="InvalidWowBasePathException"/> class.
		/// </summary>
		public InvalidWowBasePathException()
			: this(string.Empty, null, null)
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="InvalidWowBasePathException"/> class.
		/// </summary>
		/// <param name="wowBasePath">The wow base path.</param>
		public InvalidWowBasePathException(string wowBasePath)
			: this(wowBasePath, null, null)
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="InvalidWowBasePathException"/> class
		/// with a specified error message.
		/// </summary>
		/// <param name="wowBasePath">The wow base path.</param>
		/// <param name="message">The message that describes the error.</param>
		public InvalidWowBasePathException(string wowBasePath, string message)
			: this(wowBasePath, message, null)
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="InvalidWowBasePathException"/> class with a specified error message
		/// and a reference to the inner exception that is the cause of this exception.
		/// </summary>
		/// <param name="wowBasePath">The wow base path.</param>
		/// <param name="message">The message that describes the error.</param>
		/// <param name="innerException">The exception that is the cause of the current exception, or a null reference if no inner exception is specified.</param>
		public InvalidWowBasePathException(string wowBasePath, string message, Exception innerException)
			: base(message, innerException)
		{
			WowBasePath = wowBasePath;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="InvalidWowBasePathException"/> class with serialized data.
		/// </summary>
		/// <param name="info">The <see cref="System.Runtime.Serialization.SerializationInfo"/> that holds the serialized object data about the exception being thrown.</param>
		/// <param name="context">The <see cref="System.Runtime.Serialization.StreamingContext"/> that contains contextual information about the source or destination.</param>
		protected InvalidWowBasePathException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
			if (info != null)
				WowBasePath = info.GetString("WowBasePath");
		}

		/// <summary>
		/// When overridden in a derived class, sets the System.Runtime.Serialization.SerializationInfo with information about the exception.
		/// </summary>
		/// <param name="info"></param>
		/// <param name="context"></param>
		[SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter=true)]
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
			info.AddValue("WowBasePath", WowBasePath);
		}

		/// <summary>
		/// Gets a message that describes the current exception.
		/// </summary>
		/// <returns>The error message that explains the reason for the exception, or an empty string("").</returns>
		public override string Message
		{
			get
			{
				string message = base.Message;
				if (InnerException != null)
					message = String.Concat(message, "\n", InnerException.Message);
				return message;
			}
		}
	}
}