﻿using System;
using System.ComponentModel;
using System.Globalization;
using System.Text.RegularExpressions;
using Microsoft.WowAddonStudio.FrameXml.Serialization.LayoutFrames;

namespace Microsoft.WowAddonStudio.FrameXml.Serialization.TypeConverters
{
    public class FontStringTypeConverter : ExpandableObjectConverter
    {
        private readonly static ColorTypeConverter colorTypeConverter = new ColorTypeConverter();
        private readonly static FontHeightTypeConverter fontHeightTypeConverter = new FontHeightTypeConverter();

        private readonly static char listSeparator = CultureInfo.CurrentCulture.TextInfo.ListSeparator[0];
        private readonly static Regex fontStringRegex = new Regex(String.Format(@"^(?<Color>[a-zA-Z]+|([01]?\d\d?|2[0-4]\d|25[0-5])\{0}([01]?\d\d?|2[0-4]\d|25[0-5])\{0}([01]?\d\d?|2[0-4]\d|25[0-5]))\s+(?<FontHeight>(\d+))$", listSeparator), RegexOptions.Compiled);

        /// <summary>
        /// Returns whether this converter can convert an object of the given type to the type of this converter, using the specified context.
        /// </summary>
        /// <param name="context">An <see cref="T:System.ComponentModel.ITypeDescriptorContext"/> that provides a format context.</param>
        /// <param name="sourceType">A <see cref="T:System.Type"/> that represents the type you want to convert from.</param>
        /// <returns>
        /// true if this converter can perform the conversion; otherwise, false.
        /// </returns>
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            if (sourceType == typeof(String))
                return true;

            return base.CanConvertFrom(context, sourceType);
        }

        /// <summary>
        /// Returns whether this converter can convert the object to the specified type, using the specified context.
        /// </summary>
        /// <param name="context">An <see cref="T:System.ComponentModel.ITypeDescriptorContext"/> that provides a format context.</param>
        /// <param name="destinationType">A <see cref="T:System.Type"/> that represents the type you want to convert to.</param>
        /// <returns>
        /// true if this converter can perform the conversion; otherwise, false.
        /// </returns>
        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            if (destinationType == typeof(FontString))
                return true;

            return base.CanConvertTo(context, destinationType);
        }

        /// <summary>
        /// Converts the given object to the type of this converter, using the specified context and culture information.
        /// </summary>
        /// <param name="context">An <see cref="T:System.ComponentModel.ITypeDescriptorContext"/> that provides a format context.</param>
        /// <param name="culture">The <see cref="T:System.Globalization.CultureInfo"/> to use as the current culture.</param>
        /// <param name="value">The <see cref="T:System.Object"/> to convert.</param>
        /// <returns>
        /// An <see cref="T:System.Object"/> that represents the converted value.
        /// </returns>
        /// <exception cref="T:System.NotSupportedException">
        /// The conversion cannot be performed.
        /// </exception>
        public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
        {
            if (value is String)
            {
                var fontStringValue = (string)value;

                if (String.IsNullOrEmpty(fontStringValue))
                    return null;

                if (fontStringRegex.IsMatch(fontStringValue))
                {
                    var match = fontStringRegex.Match(fontStringValue);

                    string colorValue = match.Groups["Color"].Value;
                    string fontHeightValue = match.Groups["FontHeight"].Value;

                    Color color = (Color)colorTypeConverter.ConvertFrom(colorValue);
                    FontHeight fontHeight = (FontHeight)fontHeightTypeConverter.ConvertFrom(fontHeightValue);

                    return new FontString { FontHeight = fontHeight, Color = color };
                }

                throw new ArgumentException(String.Format(CultureInfo.InvariantCulture, "Invalid FontString. Use one of the following format: R{0}G{0}B [space] 'font height' or 'color name' [space] 'font height'", listSeparator), "value");
            }

            return base.ConvertFrom(context, culture, value);
        }

        /// <summary>
        /// Converts the given value object to the specified type, using the specified context and culture information.
        /// </summary>
        /// <param name="context">An <see cref="T:System.ComponentModel.ITypeDescriptorContext"/> that provides a format context.</param>
        /// <param name="culture">A <see cref="T:System.Globalization.CultureInfo"/>. If null is passed, the current culture is assumed.</param>
        /// <param name="value">The <see cref="T:System.Object"/> to convert.</param>
        /// <param name="destinationType">The <see cref="T:System.Type"/> to convert the <paramref name="value"/> parameter to.</param>
        /// <returns>
        /// An <see cref="T:System.Object"/> that represents the converted value.
        /// </returns>
        /// <exception cref="T:System.ArgumentNullException">
        /// The <paramref name="destinationType"/> parameter is null.
        /// </exception>
        /// <exception cref="T:System.NotSupportedException">
        /// The conversion cannot be performed.
        /// </exception>
        public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
        {
            if ((destinationType == typeof(String)) && (value is FontString))
            {
                var fontString = (FontString)value;

                string colorValue = (string)colorTypeConverter.ConvertTo(fontString.Color, typeof(string));
                string offsetValue = (string)fontHeightTypeConverter.ConvertTo(fontString.FontHeight, typeof(string));

                return String.Format("{0} {1}", colorValue, offsetValue);
            }

            return base.ConvertTo(context, culture, value, destinationType);
        }
    }
}
