using System;
using System.ComponentModel;
using System.Globalization;
using System.Text.RegularExpressions;

namespace Microsoft.WowAddonStudio.FrameXml.Serialization.TypeConverters
{
	/// <summary>
	/// Provides a type converter to convert expandable objects to and from various other representations.
	/// </summary>
	public class InsetTypeConverter : ExpandableObjectConverter
	{
		private static readonly Regex insetStringRegex;

		private static readonly char listSeparator = CultureInfo.CurrentCulture.TextInfo.ListSeparator[0];

		/// <summary>
		/// Initializes the <see cref="InsetTypeConverter"/> class.
		/// </summary>
		static InsetTypeConverter()
		{
            string regExstring = String.Concat(@"^(-?(\d+)", listSeparator, @"){3}(-?(\d+)", listSeparator, @"{0,1}){1}$");
			insetStringRegex = new Regex(regExstring, RegexOptions.Compiled);
		}

		/// <summary>
		/// Returns whether this converter can convert the object to the specified type, using the specified context.
		/// </summary>
		/// <param name="context">An <see cref="T:System.ComponentModel.ITypeDescriptorContext"/> that provides a format context.</param>
		/// <param name="destinationType">A <see cref="T:System.Type"/> that represents the type you want to convert to.</param>
		/// <returns>
		/// true if this converter can perform the conversion; otherwise, false.
		/// </returns>
		public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
		{
			if (destinationType == typeof (Inset))
				return true;

			return base.CanConvertTo(context, destinationType);
		}

		/// <summary>
		/// Returns whether this converter can convert an object of the given type to the type of this converter, using the specified context.
		/// </summary>
		/// <param name="context">An <see cref="T:System.ComponentModel.ITypeDescriptorContext"/> that provides a format context.</param>
		/// <param name="sourceType">A <see cref="T:System.Type"/> that represents the type you want to convert from.</param>
		/// <returns>
		/// true if this converter can perform the conversion; otherwise, false.
		/// </returns>
		public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
		{
			if (sourceType == typeof (string))
				return true;

			return base.CanConvertFrom(context, sourceType);
		}

		/// <summary>
		/// Converts the given value object to the specified type, using the specified context and culture information.
		/// </summary>
		/// <returns>
		/// An <see cref="T:System.Object"></see> that represents the converted value.
		/// </returns>
		/// <param name="culture">A <see cref="T:System.Globalization.CultureInfo"></see>. If null is passed, the current culture is assumed. </param>
		/// <param name="context">An <see cref="T:System.ComponentModel.ITypeDescriptorContext"></see> that provides a format context. </param>
		/// <param name="destinationType">The <see cref="T:System.Type"></see> to convert the value parameter to. </param>
		/// <param name="value">The <see cref="T:System.Object"></see> to convert. </param>
		/// <exception cref="T:System.NotSupportedException">The conversion cannot be performed. </exception>
		/// <exception cref="T:System.ArgumentNullException">The destinationType parameter is null. </exception>
		public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value,
		                                 Type destinationType)
		{
			if ((destinationType == typeof (String)) && (value is Inset))
			{
				var inset = (Inset) value;
				return String.Format(culture, "{0}{4}{1}{4}{2}{4}{3}", inset.Left, inset.Right, inset.Top, inset.Bottom,
				                     listSeparator);
			}
			return base.ConvertTo(context, culture, value, destinationType);
		}

		/// <summary>
		/// Converts the given object to the type of this converter, using the specified context and culture information.
		/// </summary>
		/// <returns>
		/// An <see cref="T:System.Object"></see> that represents the converted value.
		/// </returns>
		/// <param name="culture">The <see cref="T:System.Globalization.CultureInfo"></see> to use as the current culture. </param>
		/// <param name="context">An <see cref="T:System.ComponentModel.ITypeDescriptorContext"></see> that provides a format context. </param>
		/// <param name="value">The <see cref="T:System.Object"></see> to convert. </param>
		/// <exception cref="T:System.NotSupportedException">The conversion cannot be performed. </exception>
		public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
		{
			if (value is string)
			{
				var textValue = (string) value;
				if (!string.IsNullOrEmpty(textValue))
				{
					Match match = insetStringRegex.Match(textValue);

					if (!match.Success)
					{
						int singleValue;
						if (Int32.TryParse(textValue, NumberStyles.Any, culture, out singleValue))
						{
							return new Inset(singleValue);
						}
						throw new ArgumentException(String.Format(CultureInfo.InvariantCulture, "Invalid Inset. (use: [Left]{0}[Right]{0}[Top]{0}[Bottom])", listSeparator));
					}

					string[] insetValues = textValue.Split(listSeparator);
					return new Inset(int.Parse(insetValues[0]), int.Parse(insetValues[1]), int.Parse(insetValues[2]), int.Parse(insetValues[3]));
				}
                return null;
			}
			return base.ConvertFrom(context, culture, value);
		}
	}
}