using System;
using System.Runtime.Serialization;
using System.Security.Permissions;

namespace Microsoft.WowAddonStudio.Lua.RuntimeService.Common
{
	/// <summary>
	/// Represents errors that occur during LuaRuntime execution.
	/// </summary>
	[Serializable]
	public class LuaRuntimeException : Exception
	{
		/// <summary>
		/// Gets the name of the runtime which caused the exception.
		/// </summary>
		public string RuntimeName { get; protected set; }

		/// <summary>
		/// Initializes a new instance of the <see cref="LuaRuntimeException"/> class.
		/// </summary>
		/// <param name="runtime">The <see cref="LuaRuntime"/> parameter.</param>
		public LuaRuntimeException(ILuaRuntime runtime)
			: this(runtime, null, null)
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="LuaRuntimeException"/> class
		///  with a specified error message.
		/// </summary>
		/// <param name="runtime">The <see cref="LuaRuntime"/> parameter.</param>
		/// <param name="message">The message that describes the error.</param>
		public LuaRuntimeException(ILuaRuntime runtime, string message)
			: this(runtime, message, null)
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="LuaRuntimeException"/> class with a specified error message
		/// and a reference to the inner exception that is the cause of this exception.
		/// </summary>
		/// <param name="runtime">The <see cref="LuaRuntime"/> parameter.</param>
		/// <param name="message">The message that describes the error.</param>
		/// <param name="innerException">The exception that is the cause of the current exception, or a null reference if no inner exception is specified.</param>
		public LuaRuntimeException(ILuaRuntime runtime, string message, Exception innerException)
			: base(message, innerException)
		{
			if (runtime != null)
				RuntimeName = runtime.Name;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="LuaRuntimeException"/> class with serialized data.
		/// </summary>
		/// <param name="info">The <see cref="System.Runtime.Serialization.SerializationInfo"/> that holds the serialized object data about the exception being thrown.</param>
		/// <param name="context">The <see cref="System.Runtime.Serialization.StreamingContext"/> that contains contextual information about the source or destination.</param>
		protected LuaRuntimeException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
			if (info != null)
				RuntimeName = info.GetString("RuntimeName");
		}

		/// <summary>
		/// When overridden in a derived class, sets the System.Runtime.Serialization.SerializationInfo with information about the exception.
		/// </summary>
		/// <param name="info"></param>
		/// <param name="context"></param>
		[SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter=true)]
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
			info.AddValue("RuntimeName", RuntimeName);
		}

		/// <summary>
		/// Gets a message that describes the current exception.
		/// </summary>
		/// <returns>The error message that explains the reason for the exception, or an empty string("").</returns>
		public override string Message
		{
			get
			{
				string message = base.Message;
				if (InnerException != null)
					message = String.Concat(message, "\n", InnerException.Message);
				return message;
			}
		}
	}
}