using System;
using System.Runtime.Serialization;
using System.Security.Permissions;

namespace Microsoft.WowAddonStudio.Lua.RuntimeService.Common
{
	/// <summary>
	/// Represents errors that occur during LuaRuntime execution.
	/// </summary>
	[Serializable]
	public class LuaRuntimeInitializationException : LuaRuntimeException
	{
		public string Argument { get; protected set; }

		/// <summary>
		/// Initializes a new instance of the <see cref="LuaRuntimeInitializationException"/> class.
		/// </summary>
		/// <param name="runtime">The <see cref="LuaRuntime"/> parameter.</param>
		public LuaRuntimeInitializationException(ILuaRuntime runtime)
			: this(runtime, null, null)
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="LuaRuntimeInitializationException"/> class
		///  with a specified error message.
		/// </summary>
		/// <param name="runtime">The <see cref="LuaRuntime"/> parameter.</param>
		/// <param name="message">The message that describes the error.</param>
		public LuaRuntimeInitializationException(ILuaRuntime runtime, string message)
			: this(runtime, message, new ArgumentNullException(message))
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="LuaRuntimeInitializationException"/> class with a specified error message
		/// and a reference to the inner exception that is the cause of this exception.
		/// </summary>
		/// <param name="runtime">The <see cref="LuaRuntime"/> parameter.</param>
		/// <param name="message">The message that describes the error.</param>
		/// <param name="innerException">The exception that is the cause of the current exception, or a null reference if no inner exception is specified.</param>
		public LuaRuntimeInitializationException(ILuaRuntime runtime, string message, Exception innerException)
			: base(runtime, message, innerException)
		{
			Argument = message;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="LuaRuntimeInitializationException"/> class with serialized data.
		/// </summary>
		/// <param name="info">The <see cref="System.Runtime.Serialization.SerializationInfo"/> that holds the serialized object data about the exception being thrown.</param>
		/// <param name="context">The <see cref="System.Runtime.Serialization.StreamingContext"/> that contains contextual information about the source or destination.</param>
		protected LuaRuntimeInitializationException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
			if (info != null)
				Argument = info.GetString("Argument");
		}


		/// <summary>
		/// When overridden in a derived class, sets the System.Runtime.Serialization.SerializationInfo with information about the exception.
		/// </summary>
		/// <param name="info"></param>
		/// <param name="context"></param>
		[SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter = true)]
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
			info.AddValue("Argument", Argument);
		}
	}
}