%namespace Microsoft.WowAddonStudio.Lua.Parser

%x BLOCKCOMMENT,DQSTRING,SQSTRING,LBSTRING

%{
	 // Variable holding the current string literal's block level
	 int blockLevel;

	 // Dictionary mapping the Lua keywords to their token values
	 private static Dictionary<string, Tokens> tokenMap = new Dictionary<string, Tokens>()
														{
															{ "and", Tokens.KWAND },
															{ "break", Tokens.KWBREAK },
															{ "do", Tokens.KWDO },
															{ "end", Tokens.KWEND },
															{ "else", Tokens.KWELSE },
															{ "elseif", Tokens.KWELSEIF },
															{ "for", Tokens.KWFOR },
															{ "false", Tokens.KWFALSE },
															{ "function", Tokens.KWFUNCTION },
															{ "if", Tokens.KWIF },
															{ "in", Tokens.KWIN },
															{ "local", Tokens.KWLOCAL },
															{ "nil", Tokens.KWNIL },
															{ "not", Tokens.KWNOT },
															{ "or", Tokens.KWOR },
															{ "repeat", Tokens.KWREPEAT },
															{ "return", Tokens.KWRETURN },
															{ "then", Tokens.KWTHEN },
															{ "true", Tokens.KWTRUE },
															{ "until", Tokens.KWUNTIL },
															{ "while", Tokens.KWWHILE }
														};
	 
	 
	private int GetToken(string tokenText)
    {
		if (tokenText != null && tokenMap.ContainsKey(tokenText))        
			return (int)tokenMap[tokenText];

		return (int)Tokens.IDENTIFIER;
	}

	public override void yywarn(string message, params object[] args)
	{
		if (ErrorHandler != null) 
			ErrorHandler.AddError(String.Format(message, args), tokLin, tokCol, tokECol - tokCol, true);
	}

	public override void yyerror(string message, params object[] args)
	{
		if (ErrorHandler != null) 
			ErrorHandler.AddError(String.Format(message, args), tokLin, tokCol, tokECol - tokCol, false);
	}
%}

Anything				.*
ABOpenBrkt				[^\[=]
ABCloseBrkt				[^\]]
AllowSingleOpenBrkt		\[?
AllowSingleCloseBrkt	\]?

BlkStart		    \[=*\[
BlkEnd				\]=*\]

CmntStart			--
CmntEnd				\n

LongCmntStart		--{BlkStart}

Whitespace      [ \t\r\f\v\n]

%%

\+                        { return (int)Tokens.PLUS;    }
\-                        { return (int)Tokens.MINUS;    }
\*                        { return (int)Tokens.ASTERISK;    }
\/                        { return (int)Tokens.SLASH;    }
\%                        { return (int)Tokens.PERCENT;    }
\^                        { return (int)Tokens.CARET;    }
\#                        { return (int)Tokens.POUND;    }
=                         { return (int)Tokens.EQUAL;    }
==                        { return (int)Tokens.EQ;  }
~=						  { return (int)Tokens.NEQ; }
\<=                       { return (int)Tokens.LTE;    }
\>=                       { return (int)Tokens.GTE;    }
\<                        { return (int)Tokens.LT;     }
\>                        { return (int)Tokens.GT; }

\(                        { return (int)Tokens.LPAREN;    }
\)                        { return (int)Tokens.RPAREN;    }
\{                        { return (int)Tokens.LBRACE;    }
\}                        { return (int)Tokens.RBRACE;    }
\[                        { return (int)Tokens.LBRACKET;    }
\]                        { return (int)Tokens.RBRACKET;    }
;                         { return (int)Tokens.SEMICOLON;    }
:                         { return (int)Tokens.COLON;    }
,                         { return (int)Tokens.COMMA;    }
\.                        { return (int)Tokens.DOT;    }
\.\.					  { return (int)Tokens.DOTDOT; }
\.\.\.					  { return (int)Tokens.ELLIPSIS; }

/* Keywords, identifiers and primitive tokens */
[a-zA-Z_][a-zA-Z0-9_]*    { return GetToken(yytext); }

[0-9]*\.?[0-9]*((e|E)\-?[0-9]+)? { return (int)Tokens.NUMBER; }
0x[0-9a-f]+                      { return (int)Tokens.NUMBER; }

/* Single-line comments */
{CmntStart}{CmntEnd}												{ return (int)Tokens.LEX_LINECOMMENT; }
{CmntStart}{AllowSingleOpenBrkt}{ABOpenBrkt}{Anything}{CmntEnd}		{ return (int)Tokens.LEX_LINECOMMENT; }

/* Block comments */
{LongCmntStart}											{ BEGIN(BLOCKCOMMENT); blockLevel = yytext.Length - 4; return (int)Tokens.LEX_COMMENT; }
<BLOCKCOMMENT>{AllowSingleCloseBrkt}{ABCloseBrkt}*		{ return (int)Tokens.LEX_COMMENT; }
<BLOCKCOMMENT>{BlkEnd}									{ if (blockLevel == yytext.Length - 2) { BEGIN(INITIAL); } return (int)Tokens.LEX_COMMENT; }

/* Single-quoted string literals */
'														{ BEGIN(SQSTRING); return (int)Tokens.STRING; }
<SQSTRING>[^'\\\n]*										|
<SQSTRING>\\['abfnrtv\\\n]	    						|
<SQSTRING>\\[0-9]{1,3}									{ return (int)Tokens.STRING; }
<SQSTRING>\n											|
<SQSTRING>'												{ BEGIN(INITIAL); return (int)Tokens.STRING; }
<SQSTRING>\\[^'abfnrtv\\\n0-9]{0,1}						{ yywarn("Illegal escape sequence: '{0}'.", yytext); return (int)Tokens.STRING; }

/* Double-quoted string literals */
\"														{ BEGIN(DQSTRING); return (int)Tokens.STRING; }
<DQSTRING>[^\"\\\n]*									|
<DQSTRING>\\[\"abfnrtv\\\n]								|
<DQSTRING>\\[0-9]{1,3}									{ return (int)Tokens.STRING; }
<DQSTRING>\n											|
<DQSTRING>\"											{ BEGIN(INITIAL); return (int)Tokens.STRING; }
<DQSTRING>\\[^\"abfnrtv\\\n0-9]{0,1}					{ yywarn("Illegal escape sequence: '{0}'.", yytext); return (int)Tokens.STRING; }

/* Long-bracket string literals */
{BlkStart}												{ BEGIN(LBSTRING); blockLevel = yytext.Length - 2;	return (int)Tokens.STRING; }
<LBSTRING>{AllowSingleCloseBrkt}{ABCloseBrkt}           { return (int)Tokens.STRING; }
<LBSTRING>{BlkEnd}										{ if (blockLevel == yytext.Length - 2) { BEGIN(INITIAL); } return (int)Tokens.STRING; }

/* Whitespace and catch-all for illegal characters */
{Whitespace}+       { return (int)Tokens.LEX_WHITESPACE; }
\n                  { return (int)Tokens.LEX_WHITESPACE; }
.                   { yyerror("Illegal character: '{0}'", yytext);
				      return (int)Tokens.LEX_ERROR; }

%{
	   // Load the token text as the string member of the 'union'
       yylval.Text = yytext;
       
       // Initialize the current location in yylloc
       yylloc = new LexLocation(yyline - 1, yycol, yyline - 1, yycol + yyleng);
%}

%%

/* .... */
