﻿%namespace Microsoft.WowAddonStudio.Lua.Parser

%using System.Diagnostics

%using Microsoft.WowAddonStudio.Lua.AST
%using Microsoft.WowAddonStudio.Lua.Model

%YYLTYPE LexLocation
%YYSTYPE LexValue

%partial

%token <Text> KWAND KWBREAK KWDO KWEND KWELSE KWELSEIF KWFOR KWFALSE KWFUNCTION KWIF
%token <Text> KWIN KWLOCAL KWNOT KWNIL KWOR KWREPEAT KWRETURN KWTHEN KWTRUE KWUNTIL KWWHILE

%token LPAREN RPAREN LBRACE RBRACE LBRACKET RBRACKET SEMICOLON COMMA DOTDOT

%token <Text> EQUAL PLUS MINUS ASTERISK SLASH PERCENT CARET POUND ELLIPSIS
%token <Text> EQ NEQ GT GTE LT LTE

%token <Text> IDENTIFIER NUMBER STRING DOT COLON
%type <Text> UnaryOperator BinaryOperator

%type <Text> String

%type <Identifier> Identifier
%type <QualifiedName> QualifiedName FunctionName

%type <Node> Chunk Block
%type <Node> StatementList Statement LastStatement 
%type <Node> VariableList Variable 
%type <Node> ExpressionList23 ExpressionList Expression
%type <Node> Function FunctionCall PrefixExpression Arguments
%type <Node> TableConstructor
%type <Node> FieldList Field
%type <Node> IdentifierList
%type <Node> ParameterList
%type <Node> ThenBlock ElseIfBlockList

%left  ASTERISK SLASH PERCENT
%left  PLUS MINUS
%left  GT GTE LT LTE EQ NEQ
%right DOTDOT
%right KWNOT POUND
%right CARET

%token maxParseToken 
%token LEX_WHITESPACE LEX_LINECOMMENT LEX_COMMENT LEX_ERROR

%%

/* See http://lua-users.org/wiki/LuaGrammar */

Chunk
    : Block
    {
		$$ = new Chunk(@$) { Block = (Block)$1 };
		Chunk = (Chunk)$$;
    }
    ;
    
Semicolon
	: SEMICOLON
	| /* empty */
	;

Block 
	: { EnterScope(); } StatementList { LeaveScope(); }
	{
		@$ = @2;
		$$ = new Block(@$) { StatementList = (Statement)$2 };
		
		DefineScope(@$);

	}
	| {EnterScope(); } StatementList LastStatement { LeaveScope(); } Semicolon
	{
		@$ = Merge(@2, @3);
		$$ = new Block(@$) { StatementList = (Statement)AppendNode($2, $3) };
		
		DefineScope(@$);
	}
	;

StatementList
	: /* empty */
	| StatementList Statement Semicolon
	{
		@$ = Merge(@1, @2);
		$$ = AppendNode($1, $2);
	}
	;

LastStatement
	: KWBREAK
	{
		$$ = new Break(@$);
	}
	| KWRETURN
	{
		$$ = new Return(@$);
	}
	| KWRETURN ExpressionList
	{
		$$ = new Return(@$) { ExpressionList = $2 };
	}
	;
	
Statement
	: VariableList EQUAL ExpressionList
	{
		$$ = new Assignment(@$) { VariableList = $1, ExpressionList = $3 };
	}
	| FunctionCall
	{
		$$ = $1;
	}
	| KWIF Expression KWTHEN ThenBlock KWEND
	{
		$$ = new If(@$) { Expression = $2, ThenBlock = (ThenBlock)$4 };
		
		DefineScope(@3, @5);
	}
	| KWDO Block KWEND
	{
		$$ = new ExplicitBlock(@$) { Block = (Block)$2 };
		
		DefineScope(@1, @3);
		Region(@1, @3);
	}
	| KWWHILE Expression KWDO Block KWEND
	{
		$$ = new WhileLoop(@$) { Expression = $2, Block = (Block)$4 };
		
		DefineScope(@3, @5);
	}
	| KWREPEAT Block KWUNTIL Expression
	{
		$$ = new RepeatUntilLoop(@$) { Expression = $4, Block = (Block)$2 };
		
		DefineScope(@1, @3);
	}
	| KWFOR Identifier EQUAL ExpressionList23 KWDO Block KWEND
	{
		$$ = new ForLoop(@$) { Identifier = $2, Expression = $4, Block = (Block)$6 };
		
		DefineScope(@5, @7);
		Region(@5, @7);
	}
	| KWFOR IdentifierList KWIN ExpressionList KWDO Block KWEND
	{
		$$ = new ForLoop(@$) { IdentifierList = (Identifier)$2, Expression = $4, Block = (Block)$6 };
		
		DefineScope(@5, @7);
		Region(@5, @7);
	}
	| KWFUNCTION FunctionName ParameterList Block KWEND
	{
		$$ = new FunctionDeclaration(@$) { FunctionName = $2, ParameterList = $3 as ParameterList, Body = (Block)$4 };

		DefineScope(@3, @5);
		Region(@3, @5);
	}
	| KWLOCAL KWFUNCTION FunctionName ParameterList Block KWEND
	{
		$$ = new FunctionDeclaration(@$) { FunctionName = $3, ParameterList = $4 as ParameterList, Body = (Block)$5, IsLocal = true };
		
		DefineScope(@4, @6);
		Region(@4, @6);
	}
	| KWLOCAL IdentifierList
	{
		$$ = new LocalDeclaration(@$) { IdentifierList = (Identifier)$2 };
	}
	| KWLOCAL IdentifierList EQUAL ExpressionList
	{
		$$ = new Assignment(@$) { VariableList = $2, ExpressionList = $4, IsLocal = true };
	}
	| error
	{
	}
	;

ThenBlock
	: Block ElseIfBlockList
	{
		$$ = new ThenBlock(Merge(@1, @2)) { Block = (Block)$1, ElseIfBlockList = (ElseIfBlock)$2 };
	}
	| Block ElseIfBlockList KWELSE Block
	{
		$$ = new ThenBlock(Merge(@1, @4)) { Block = (Block)$1, ElseIfBlockList = (ElseIfBlock)$2, ElseBlock = (Block)$4 };
	}
	;
	
ElseIfBlockList
	: /* empty */
	| ElseIfBlockList KWELSEIF Expression KWTHEN Block
	{
		$$ = AppendNode($1, new ElseIfBlock(Merge(@2, @5)) { Expression = $3, Block = (Block)$5 });
	}
	;

FunctionName
	: QualifiedName { $$ = $1; }
	| QualifiedName COLON Identifier 
	{ 
		$$ = $1.QualifyMethod($3);
		
		QualifyName(@2, @3, $3.Name);
	}
	| QualifiedName COLON error
	{
		$$ = $1;
	}
	;
	
QualifiedName
	: Identifier 
	{
		$$ = new QualifiedName($1);
		
		StartName(@1, $1.Name);
	}
	| QualifiedName DOT Identifier 
	{ 
		$$ = $1.Qualify($3);
		
		QualifyName(@2, @3, $3.Name);
	}
	| QualifiedName DOT error
	{
		$$ = $1;
	}
	;
	
IdentifierList
	: Identifier
	{
		$$ = $1;
		
		StartName(@1, $1.Name);
	}
	| IdentifierList COMMA Identifier
	{
		$$ = AppendNode($1, $3);
		
		StartName(@3, $3.Name);
	}
	;

ExpressionList
	: Expression
	{
		$$ = $1;
	}
	| ExpressionList COMMA { Parameter(@2); } Expression
	{
		$$ = AppendNode($1, $4);		
	}
	;
	
ExpressionList23
	: Expression COMMA Expression
	{
		$$ = AppendNode($1, $3);
	}
	| Expression COMMA Expression COMMA Expression
	{
		$$ = AppendNodes($1, $3, $5);
	}
	;
	
UnaryOperator
	: KWNOT
	| POUND
	| MINUS
	;
	 
BinaryOperator
	: KWOR | KWAND
	| GT | GTE | LT | LTE | EQ | NEQ | DOTDOT 
	| PLUS | MINUS | ASTERISK | SLASH | PERCENT | CARET
	;

String
	: STRING
	{
		$$ = $1;
	}
	| String STRING
	{
		$$ = String.Concat($1, $2);
	}
	;
	
Expression
	: KWNIL
	{
		$$ = new Literal(@$) { Type = LuaType.Nil, Value = $1 };
	}
	| KWTRUE 
	{
		$$ = new Literal(@$) { Type = LuaType.Boolean, Value = $1 };
	}
	| KWFALSE 
	{
		$$ = new Literal(@$) { Type = LuaType.Boolean, Value = $1 };
	}
	| NUMBER 
	{
		$$ = new Literal(@$) { Type = LuaType.Number, Value = $1 };
	}
	| String
	{
		$$ = new Literal(@$) { Type = LuaType.String, Value = $1 };
	}
	| ELLIPSIS 
	{
		$$ = new Ellipsis(@$);
	}
	| Function
	{
		$$ = $1;
	}
	| PrefixExpression
	{
		$$ = $1;
	}
	| TableConstructor
	{
		$$ = $1;
	}
	| UnaryOperator Expression
	{
		$$ = new UnaryExpression(@$) { Operator = $1, Expression = $2 };
	}
	| Expression BinaryOperator Expression
	{
		$$ = new BinaryExpression(@$) { Operator = $2, LeftExpression = $1, RightExpression = $3 };
	}
	;
	
VariableList
	: Variable
	{
		$$ = $1;
	}
	| VariableList COMMA Variable
	{
		$$ = AppendNode($1, $3);
	}
	;
	
Variable
	: Identifier
	{
		$$ = $1;
		
		StartName(@1, $1.Name);
	}
	| PrefixExpression LBRACKET Expression RBRACKET
	{
		Match(@2, @4);
		
		$$ = new AST.Variable(@$) { PrefixExpression = $1, Expression = $3 };
	}
	| PrefixExpression DOT Identifier
	{
		// If the PrefixExpression is an Identifier, we can just append the scoped name
		if ($1 is Identifier)
		{
			$$ = new QualifiedName((Identifier)$1).Qualify($3);
		}
		else if ($1 is QualifiedName)
		{
			$$ = ((QualifiedName)$1).Qualify($3);
		}
		else
		{
			$$ = new AST.Variable(@$) { PrefixExpression = $1, Identifier = $3 };
		}
		
		QualifyName(@2, @3, $3.Name);
	}
	| PrefixExpression DOT error
	{
		// TODO: Should we return something?
	}
	;
	
PrefixExpression
	: Variable
	{
		$$ = $1;
	}
	| FunctionCall
	{
		$$ = $1;
	}
	| LPAREN Expression RPAREN
	{
		Match(@1, @3);
		
		$$ = $2;
	}
	;
	
FunctionCall
	: PrefixExpression Arguments
	{
		$$ = new FunctionCall(@$) { PrefixExpression = $1, Arguments = $2 };
	}
	| PrefixExpression COLON Identifier Arguments
	{
		$$ = new FunctionCall(@$) { PrefixExpression = $1, Identifier = $3, Arguments = $4 };
		
		QualifyName(@2, @3, $3.Name);
	}
	| PrefixExpression COLON Identifier error
	{
		QualifyName(@2, @3, $3.Name);
	}
	;
	
Arguments
	: StartArg error
	{
	}
	| StartArg EndArg
	{
		Match(@1, @2);
	}
	| StartArg ExpressionList EndArg
	{
		$$ = $2;
		
		Match(@1, @3);
	}
	| StartArg ExpressionList error
	{
		EndParameters(@3);
		
		Error(@3, "Unmatched parentheses.");
	}
	| TableConstructor
	{
		$$ = $1;
	}
	;

StartArg
	: LPAREN
	{
		StartParameters(@1);
	}
	;
	
EndArg
	: RPAREN
	{
		EndParameters(@1);
	}
	;
		
Function
	: KWFUNCTION ParameterList Block KWEND
	{
		$$ = new AST.Function(@$) { ParameterList = $2 as ParameterList, Body = (Block)$3 };
	
		DefineScope(@1, @4);	
		Region(@2, @4);
	}
	;
	
ParameterList
	: LPAREN RPAREN
	{
		Match(@1, @2);
	}
	| LPAREN error { Error(@1, "Unmatched parentheses."); }
	| LPAREN IdentifierList RPAREN
	{
		$$ = new ParameterList(@$) { IdentifierList = (Identifier)$2 };
		
		Match(@1, @3);
	}
	| LPAREN IdentifierList error { Error(@1, "Unmatched parentheses."); }
	| LPAREN ELLIPSIS RPAREN
	{
		Match(@1, @3);
	}
	| LPAREN ELLIPSIS error { Error(@1, "Unmatched parentheses."); }
	| LPAREN IdentifierList COMMA ELLIPSIS RPAREN
    {
		$$ = new ParameterList(@$) { IdentifierList = (Identifier)$2 };
		
		Match(@1, @5);
	}
	| LPAREN IdentifierList COMMA ELLIPSIS error { Error(@1, "Unmatched parentheses."); }
	;
	
FieldSeparator
	: COMMA
	| SEMICOLON
	;
	
TableConstructor
	: LBRACE RBRACE
	{
		$$ = new TableConstructor(@$);
		
		Match(@1, @2);
	}
	| LBRACE FieldList RBRACE
	{
		$$ = new TableConstructor(@$) { FieldList = (Field)$2 };
		
		Match(@1, @3);
		Region(@1, @3);
	}
	| LBRACE FieldList FieldSeparator RBRACE
	{
		$$ = new TableConstructor(@$) { FieldList = (Field)$2 };
		
		Match(@1, @4);
		Region(@1, @4);
	}
	;
	
FieldList
	: Field
	{
		$$ = $1;
	}
	| FieldList FieldSeparator Field
	{
		$$ = AppendNode($1, $3);
	}
	;
	
Field
	: Expression
	{
		$$ = new Field(@$) { Expression  = $1 };
	}
	| Identifier EQUAL Expression
	{
		$$ = new Field(@$) { Identifier = $1, Expression = $3 };		
	}
	| LBRACKET Expression RBRACKET EQUAL Expression
	{
		$$ = new Field(@$) { LeftExpression = $2, Expression = $5 };		
		
		Match(@1, @3);
	}
	;
	
Identifier
	: IDENTIFIER
	{
		$$ = new Identifier(@$) { Name = $1 };
	}
	;
%%
