﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

namespace Microsoft.WowAddonStudio.Build
{
	internal class TocFile
	{
		private readonly List<ValueLine> attributes;
		private readonly List<string> files;

		/// <summary>
		/// Initializes a new instance of the <see cref="TocFile"/> class.
		/// </summary>
		public TocFile()
		{
			attributes = new List<ValueLine>();
			files = new List<string>();
		}

		/// <summary>
		/// Reads the specified path.
		/// </summary>
		/// <param name="path">The path.</param>
		public void Read(string path)
		{
			using (TextReader reader = new StreamReader(path, Encoding.UTF8, true))
			{
				Read(reader);
			}
		}

		/// <summary>
		/// Writes the specified path.
		/// </summary>
		/// <param name="path">The path.</param>
		public void Write(string path)
		{
			using (TextWriter writer = new StreamWriter(path, false, Encoding.UTF8))
			{
				Write(writer);
			}
		}

		/// <summary>
		/// Writes the specified writer.
		/// </summary>
		/// <param name="writer">The writer.</param>
		public void Write(TextWriter writer)
		{
			attributes.ForEach(vl => vl.Write(writer));

			files.ForEach(s => writer.WriteLine(s));
		}

		/// <summary>
		/// Reads the specified reader.
		/// </summary>
		/// <param name="reader">The reader.</param>
		public void Read(TextReader reader)
		{
			string line;

			while ((line = reader.ReadLine()) != null)
			{
				if (line.StartsWith("##", StringComparison.OrdinalIgnoreCase))
				{
					Match m = Regex.Match(line, @"^##\s+(?'NAME'[^:]+)\s*:\s*(?'VALUE'.*)$", RegexOptions.Compiled);
					if (m.Success)
					{
						var vl = new ValueLine(m.Groups["NAME"].Value, m.Groups["VALUE"].Value);
						attributes.Add(vl);

						continue;
					}
				}

				files.Add(line);
			}
		}

		/// <summary>
		/// Gets the files.
		/// </summary>
		/// <value>The files.</value>
		public List<string> Files
		{
			get { return files; }
		}

		/// <summary>
		/// Gets or sets the <see cref="System.String"/> with the specified name.
		/// </summary>
		/// <value></value>
		public string this[string name]
		{
			get
			{
				ValueLine vl = attributes.Find(l => l.Name == name);
				if (vl == null)
					return null;

				return vl.Value;
			}
			set
			{
				ValueLine vl = attributes.Find(l => l.Name == name);
				if (vl == null)
				{
					vl = new ValueLine(name, value);

					attributes.Add(vl);
				}

				vl.Value = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		private abstract class Line
		{
			/// <summary>
			/// Writes the specified writer.
			/// </summary>
			/// <param name="writer">The writer.</param>
			public abstract void Write(TextWriter writer);
		}

		/// <summary>
		/// 
		/// </summary>
		private class SimpleLine : Line
		{
			private readonly string value;

			/// <summary>
			/// Initializes a new instance of the <see cref="SimpleLine"/> class.
			/// </summary>
			/// <param name="value">The value.</param>
			public SimpleLine(string value)
			{
				this.value = value;
			}

			/// <summary>
			/// Writes the specified writer.
			/// </summary>
			/// <param name="writer">The writer.</param>
			public override void Write(TextWriter writer)
			{
				writer.Write(value);
			}
		}

		/// <summary>
		/// 
		/// </summary>
		private class ValueLine : Line
		{
			private readonly string name;
			private string value;

			/// <summary>
			/// Initializes a new instance of the <see cref="ValueLine"/> class.
			/// </summary>
			/// <param name="name">The name.</param>
			/// <param name="value">The value.</param>
			public ValueLine(string name, string value)
			{
				this.name = name;
				this.value = value;
			}

			/// <summary>
			/// Gets the name.
			/// </summary>
			/// <value>The name.</value>
			public string Name
			{
				get { return name; }
			}

			/// <summary>
			/// Gets or sets the value.
			/// </summary>
			/// <value>The value.</value>
			public string Value
			{
				get { return value; }
				set { this.value = value; }
			}

			/// <summary>
			/// Writes the specified writer.
			/// </summary>
			/// <param name="writer">The writer.</param>
			public override void Write(TextWriter writer)
			{
				writer.Write("## ");
				writer.Write(name);
				writer.Write(": ");
				writer.WriteLine(value);
			}
		}
	}
}