﻿using System;
using System.IO;
using System.Text;
using Microsoft.Build.Framework;
using Microsoft.Build.Utilities;

namespace Microsoft.WowAddonStudio.Build
{
	/// <summary>
	/// 
	/// </summary>
	public sealed class TocGenerator : Task
	{
		private const string generatorComment =
			"Basic project properties and project files will be automatically added during deployment. Properties added by the user will be copied without changes.";

		/// <summary>
		/// Gets or sets the interface version.
		/// </summary>
		/// <value>The interface version.</value>
		public string InterfaceVersion { get; set; }
		/// <summary>
		/// Gets or sets the addon title.
		/// </summary>
		/// <value>The addon title.</value>
		public string AddonTitle { get; set; }
		/// <summary>
		/// Gets or sets the addon notes.
		/// </summary>
		/// <value>The addon notes.</value>
		public string AddonNotes { get; set; }
		/// <summary>
		/// Gets or sets the dependencies.
		/// </summary>
		/// <value>The dependencies.</value>
		public string Dependencies { get; set; }
		/// <summary>
		/// Gets or sets the saved variables.
		/// </summary>
		/// <value>The saved variables.</value>
		public string SavedVariables { get; set; }
		/// <summary>
		/// Gets or sets the saved variables per character.
		/// </summary>
		/// <value>The saved variables per character.</value>
		public string SavedVariablesPerCharacter { get; set; }
		/// <summary>
		/// Gets or sets the author.
		/// </summary>
		/// <value>The author.</value>
		public string Author { get; set; }
		/// <summary>
		/// Gets or sets the addon version.
		/// </summary>
		/// <value>The addon version.</value>
		public string AddonVersion { get; set; }
		/// <summary>
		/// Gets or sets the author email.
		/// </summary>
		/// <value>The author email.</value>
		public string AuthorEmail { get; set; }

		[Required]
		public ITaskItem InputPath { get; set; }

		/// <summary>
		/// Gets or sets the output path.
		/// </summary>
		/// <value>The output path.</value>
		[Required]
		public ITaskItem OutputPath { get; set; }

		/// <summary>
		/// Gets or sets the files.
		/// </summary>
		/// <value>The files.</value>
		public ITaskItem[] Files { get; set; }

		/// <summary>
		/// Executes this instance.
		/// </summary>
		/// <returns></returns>
		public override bool Execute()
		{
			var tf = new TocFile();

			// The file might not exist, if the project has no TOC files and a TOC entry was dynamically generated
			if (File.Exists(InputPath.ItemSpec))
			{
				using (TextReader r = new StreamReader(InputPath.ItemSpec, Encoding.UTF8, true))
				{
					tf.Read(r);
				}
			}

			using (TextWriter w = new StreamWriter(OutputPath.ItemSpec, false, Encoding.UTF8))
			{
				if (!IsFieldSet(tf, "X-AutoGenerated"))
					SetField(tf, "X-AutoGenerated", "true");
				if (!IsFieldSet(tf, "X-GeneratorComment"))
					SetField(tf, "X-GeneratorComment", generatorComment);

				SetField(tf, "Interface", InterfaceVersion);
				SetField(tf, "Title", AddonTitle);
				SetField(tf, "Notes", AddonNotes);
				SetField(tf, "Dependencies", Dependencies);
				SetField(tf, "SavedVariables", SavedVariables);
				SetField(tf, "SavedVariablesPerCharacter", SavedVariablesPerCharacter);
				SetField(tf, "Author", Author);
				SetField(tf, "Version", AddonVersion);
				SetField(tf, "X-eMail", AuthorEmail);

				tf.Files.Clear();

				if (Files != null)
				{
					foreach (ITaskItem item in Files)
					{
						tf.Files.Add(item.ItemSpec);
					}
				}

				tf.Write(w);
			}

			return true;
		}

		/// <summary>
		/// Sets the field.
		/// </summary>
		/// <param name="file">The file.</param>
		/// <param name="name">The name.</param>
		/// <param name="value">The value.</param>
		private static void SetField(TocFile file, string name, string value)
		{
			if (!String.IsNullOrEmpty(value))
				file[name] = value;
		}

		/// <summary>
		/// Determines whether [is field set] [the specified file].
		/// </summary>
		/// <param name="file">The file.</param>
		/// <param name="name">The name.</param>
		/// <returns>
		/// 	<c>true</c> if [is field set] [the specified file]; otherwise, <c>false</c>.
		/// </returns>
		private static bool IsFieldSet(TocFile file, string name)
		{
			return file[name] != null;
		}
	}
}